@@ -19,15 +19,25 @@ fi
 
 if [ -d tmp ] ; then rm -rf tmp ; fi
 mkdir tmp
-printf "testing plzip..."
 cd "${objdir}"/tmp
 
-cat "${testdir}"/test1 > in || framework_failure
+cat "${testdir}"/test.txt > in || framework_failure
 cat in in in in > in4 || framework_failure
 fail=0
 
-"${LZIP}" -cd "${testdir}"/test1.lz > copy || fail=1
+printf "testing plzip-%s..." "$2"
+
+"${LZIP}" -t "${testdir}"/test_v0.lz || fail=1
+printf .
+"${LZIP}" -cd "${testdir}"/test_v0.lz > copy || fail=1
 cmp in copy || fail=1
+printf .
+
+"${LZIP}" -t "${testdir}"/test_v1.lz || fail=1
+printf .
+"${LZIP}" -cd "${testdir}"/test_v1.lz > copy || fail=1
+cmp in copy || fail=1
+printf .
 
 for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
 	"${LZIP}" -k -$i in || fail=1
@@ -54,12 +64,17 @@ for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
 done
 
 for i in s4Ki 0 1 2 3 4 5 6 7 8 9 ; do
-	"${LZIP}" -fe -$i -o out < in || fail=1
+	"${LZIP}" -f -$i -o out < in || fail=1
 	"${LZIP}" -df -o copy < out.lz || fail=1
 	cmp in copy || fail=1
 	printf .
 done
 
+"${LZIP}" < in > anyothername || fail=1
+"${LZIP}" -d anyothername || fail=1
+cmp in anyothername.out || fail=1
+printf .
+
 for i in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 ; do
 	"${LZIP}" -s4Ki -B8Ki -n$i < in4 > out4 || fail=1
 	"${LZIP}" -d -n$i < out4 > copy4 || fail=1
